----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    11:32:54 01/10/2012 
-- Design Name: 
-- Module Name:    QuadCont - arch 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity QuadCont is
    Port ( 
			reset 		 : in  std_logic;
         clk 			 : in  std_logic;
			
			i_runQC		 : in std_logic;
			i_DGCH		 : in std_logic;
			i_selectTime : in std_logic;
			
			o_QCError	 : out std_logic
			);
end QuadCont;

architecture arch of QuadCont is

	type TA_state_type is 	(
										State_Wait,
										State_Start,
										State_QC1,
										State_QC2,
										State_Error
									);
							
	signal TA_state_now, TA_state_next: TA_state_type;
	
	signal sl_start_timer		: std_logic;
	signal sl_synch_timer		: std_logic;
	signal sl_timer_complete	: std_logic;
	
	signal sl_CH_Pos				: std_logic;
	signal sl_CH_Neg				: std_logic;
	

	constant C_DELAYWIDTH		: integer := 18;
	constant C_DELAYTIME1		: integer := 45000; -- Twitchy?
	--constant C_DELAYTIME1		: integer := 50000; -- Normal?
	constant C_DELAYTIME2		: integer := 90000;
	
	signal slv_DelayTime			: std_logic_vector(C_DELAYWIDTH-1 downto 0);
	constant C_TIMERVALUE1		: std_logic_vector(C_DELAYWIDTH-1 downto 0) := std_logic_vector(to_unsigned(C_DELAYTIME1,C_DELAYWIDTH));
	constant C_TIMERVALUE2		: std_logic_vector(C_DELAYWIDTH-1 downto 0) := std_logic_vector(to_unsigned(C_DELAYTIME2,C_DELAYWIDTH));

begin

		-- Timer to give 1ms delay.
		QuadCont_timer: entity work.Generic_Timer(arch)
		generic map (N=>C_DELAYWIDTH)
		port map   (
				clk => clk,
				reset => reset,
				start => sl_start_timer,
				en => '1',
				synch_clear => sl_synch_timer,
				time_max_in => slv_DelayTime,
				time_current_out => open,
				tick => sl_timer_complete
				);
				
		CH_POS: entity work.edge_detector(v0)
		port map(
			I_rst			=> reset,
			I_clk			=> clk,
			--
			I_enable		=> '1',
			I_edge_type	=> '1',
			--
			I_data_in	=> i_DGCH,
			O_edge_out	=> sl_CH_Pos
		);
		
		CH_NEG: entity work.edge_detector(v0)
		port map(
			I_rst			=> reset,
			I_clk			=> clk,
			--
			I_enable		=> '1',
			I_edge_type	=> '0',
			--
			I_data_in	=> i_DGCH,
			O_edge_out	=> sl_CH_Neg
		);

		
	-- Processes
	process(clk, reset)
			
	begin
			
		if (reset = '1') then
			TA_state_now <= State_Wait;
		elsif (clk'event and clk = '1') then
			TA_state_now <= TA_state_next;
		end if;
				
	end process;
	
	process(i_selectTime)
	begin
	
		if (i_selectTime = '1') then
			slv_DelayTime <= C_TIMERVALUE2;
		else
			slv_DelayTime <= C_TIMERVALUE1;
		end if;
	
	end process;
	
	process(TA_state_now, TA_state_next, i_runQC,
		sl_timer_complete, sl_CH_Pos, sl_CH_Neg)
	
	begin
	
		TA_state_next 	<= TA_state_now;
		sl_start_timer	<= '0';
		sl_synch_timer <= '0';	
		o_QCError	<= '0';
		
		case TA_state_now is
		
			when State_Wait =>
					
				TA_state_next <= State_Wait;
				if (i_runQC = '1') then 
					TA_state_next <= State_Start;
				end if;
				
			when State_Start =>
				TA_state_next <= State_QC1;
				-- Start the Timer
				sl_start_timer <= '1';
				
			when State_QC1 =>
				TA_state_next <= State_QC1;
				-- If QC check is no longer required.
				if (i_runQC = '0') then 
					TA_state_next <= State_QC2;
					sl_synch_timer <= '1';	
				-- Now we want to see a change in the DG Channel signal before the timer
				-- Times out
				elsif (sl_CH_Pos = '1' or sl_CH_Neg = '1') then
					-- Reset the Timer
					sl_synch_timer <= '1';
					-- Now need to wait at least 2 clock cycles before we can restart the timer
					-- so jump to state QC2 -> Wait
					TA_state_next <= State_QC2;
				-- Timer Times Out
				elsif (sl_timer_complete = '1') then
					TA_state_next <= State_Error;
					
				end if;
				
			when State_QC2 =>
				TA_state_next <= State_Wait;
				
			when State_Error =>
				-- Wait until i_runQC drops to 0 before resetting state machine
				o_QCError <= '1';
				if (i_runQC = '0') then
					TA_state_next <= State_Wait;
				end if;
			
		end case;
		
	end process;

end arch;

