----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    10:35:02 03/07/2010 
-- Design Name: 
-- Module Name:    Actuation_Data - arch
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity Actuation_Data is
    Port ( 
		AD_I_clk 				: in STD_LOGIC;
      AD_I_reset 				: in STD_LOGIC;
		AD_I_CaptureData		: in std_logic;
		AD_I_ActuateTrig		: in std_logic;
		AD_I_USBAcknowledge	: in std_logic;
		
		AD_I_StartDepth		: in std_logic_vector(8 downto 0);
		AD_I_MinDepth			: in std_logic_vector(8 downto 0);
		AD_I_Count				: in std_logic_vector(23 downto 0);
		
		AD_I_SP1Time			: in std_logic_vector(31 downto 0);
		AD_I_MinTime			: in std_logic_vector(31 downto 0);
		AD_I_ActTime			: in std_logic_vector(31 downto 0);
		
		AD_I_QuadErrorValue	: in std_logic_vector(8 downto 0);
		AD_I_QuadErrorCode	: in std_logic_vector(2 downto 0);
		AD_I_QuadFlag			: in std_logic;
		
		AD_I_UnderCapture		: in std_logic;
		AD_I_OverCapture		: in std_logic;
		AD_I_UnderOverTrig	: in std_logic;
		
		AD_I_SP1Locked			: in std_logic_vector(8 downto 0);
		AD_I_SP2					: in std_logic_vector(8 downto 0);
				
		-- BPS Stuff
		AD_I_BPSErrors					: in std_logic_vector(31 downto 0);
		
		AD_I_BPSSp1MaxTime			: in std_logic_vector(31 downto 0);
		AD_I_BPSSp1MinTime			: in std_logic_vector(31 downto 0);
		AD_I_BPSMinPosMaxTime		: in std_logic_vector(31 downto 0);
		AD_I_BPSMinPosMinTime		: in std_logic_vector(31 downto 0);
		AD_I_BPSMinPosMaxValue		: in std_logic_vector(31 downto 0);
		AD_I_BPSMinPosMinValue		: in std_logic_vector(31 downto 0);
		AD_I_BPSActMaxTime			: in std_logic_vector(31 downto 0);
		AD_I_BPSActMinTime			: in std_logic_vector(31 downto 0);
		
		AD_O_ActuationPos				: out std_logic_vector(31 downto 0);
		AD_O_ActuationCount 			: out std_logic_vector(31 downto 0);
		AD_O_ActuationError  		: out std_logic_vector(31 downto 0);
		AD_O_SetPoints					: out std_logic_vector(31 downto 0);
		
		AD_O_SP1Time					: out std_logic_vector(31 downto 0);
		AD_O_MinTime					: out std_logic_vector(31 downto 0);
		AD_O_ActTime					: out std_logic_vector(31 downto 0);
		
		AD_O_BPSErrors					: out std_logic_vector(31 downto 0);
		
		AD_O_BPSSp1MaxTime			: out std_logic_vector(31 downto 0);
		AD_O_BPSSp1MinTime			: out std_logic_vector(31 downto 0);
		AD_O_BPSMinPosMaxTime		: out std_logic_vector(31 downto 0);
		AD_O_BPSMinPosMinTime		: out std_logic_vector(31 downto 0);
		AD_O_BPSMinPosMaxValue		: out std_logic_vector(31 downto 0);
		AD_O_BPSMinPosMinValue		: out std_logic_vector(31 downto 0);
		AD_O_BPSActMaxTime			: out std_logic_vector(31 downto 0);
		AD_O_BPSActMinTime			: out std_logic_vector(31 downto 0);
		
		-- SPI
		AD_I_CHA_Complete				: in std_logic;
		AD_I_CHB_Complete				: in std_logic;
		AD_I_INDX_Complete			: in std_logic;
		
		AD_I_CHA_Mon					: in std_logic_vector(15 downto 0);
		AD_I_CHB_Mon					: in std_logic_vector(15 downto 0);
		AD_I_INDX_Mon					: in std_logic_vector(15 downto 0);
		
		AD_O_CHA_Mon					: out std_logic_vector(31 downto 0);
		AD_O_CHB_Mon					: out std_logic_vector(31 downto 0);
		AD_O_INDX_Mon					: out std_logic_vector(31 downto 0);
		
		AD_I_Velocity					: in  std_logic_vector(31 downto 0);
		AD_O_Velocity					: out std_logic_vector(31 downto 0);
		
		AD_O_DataGood					: out std_logic;
		AD_O_FIFOFull					: out std_logic		
		);
		
end Actuation_Data;

architecture arch of Actuation_Data is

	signal SA_ActuationPosFI		: std_logic_vector(31 downto 0);
	signal SA_ActuationErrorFI		: std_logic_vector(31 downto 0);
	signal SA_ActuationCountFI		: std_logic_vector(31 downto 0);
	signal SA_ActuationSPFI			: std_logic_vector(31 downto 0);
	signal SA_SP1TimeFI				: std_logic_vector(31 downto 0);
	signal SA_MinTimeFI				: std_logic_vector(31 downto 0);
	signal SA_ActTimeFI				: std_logic_vector(31 downto 0);

	signal SA_QuadErrorCodeNow		: std_logic_vector(2 downto 0);
	signal SA_QuadErrorCodeNext	: std_logic_vector(2 downto 0);
	signal SA_QuadErrorValueNow	: std_logic_vector(8 downto 0);
	signal SA_QuadErrorValueNext	: std_logic_vector(8 downto 0);
	
	signal SA_OverCountNow			: std_logic;
	signal SA_OverCountNext			: std_logic;
	signal SA_UnderCountNow			: std_logic;
	signal SA_UnderCountNext		: std_logic;
	
	signal SA_SP1LockedNow			: std_logic_vector(8 downto 0);
	signal SA_SP1LockedNext			: std_logic_vector(8 downto 0);
	signal SA_SP2Now					: std_logic_vector(8 downto 0);
	signal SA_SP2Next					: std_logic_vector(8 downto 0);
	
	signal SA_AcknowledgeDG			: std_logic;
	
	signal SA_readFIFO				: std_logic;
	signal SA_writeFIFO				: std_logic;
	
	signal SA_FIFOfull				: std_logic;
	signal SA_FIFOempty				: std_logic;

	constant FIFO_DEPTH				: integer := 2;
	
	signal SA_ChA_Mon_Now			: std_logic_vector(31 downto 0);
	signal SA_ChA_Mon_Next			: std_logic_vector(31 downto 0);
	signal SA_ChB_Mon_Now			: std_logic_vector(31 downto 0);
	signal SA_ChB_Mon_Next			: std_logic_vector(31 downto 0);
	signal SA_Indx_Mon_Now			: std_logic_vector(31 downto 0);
	signal SA_Indx_Mon_Next			: std_logic_vector(31 downto 0);
	
	signal SA_Velocity				: std_logic_vector(31 downto 0);
	
begin

	-- Instantiate FIFOs
	-- We only need the full and empty signals off of one FIFO as the others
	-- are synchronised to the first FIFO
	
	--Not that the FIFO are written when the signal CAP_O_AquireActData goes high at the
	--end of a capture
	
	fifo_ActPos: entity work.fifo(arch)
		generic map (B => 32, W => FIFO_DEPTH)
		port map(
			clk			=> AD_I_clk,
			reset			=> AD_I_reset,
			rd				=> SA_readFIFO,
			wr				=> SA_writeFIFO,
			w_data		=> SA_ActuationPosFI,
			empty			=> SA_FIFOempty,
			full			=> SA_FIFOfull,
			r_data		=>	AD_O_ActuationPos
			);
			
	fifo_ActError: entity work.fifo(arch)
		generic map (B => 32, W => FIFO_DEPTH)
		port map(
			clk			=> AD_I_clk,
			reset			=> AD_I_reset,
			rd				=> SA_readFIFO,
			wr				=> SA_writeFIFO,
			w_data		=> SA_ActuationErrorFI,
			empty			=> open,
			full			=> open,
			r_data		=>	AD_O_ActuationError
			);
			
	fifo_ActCount: entity work.fifo(arch)
		generic map (B => 32, W => FIFO_DEPTH)
		port map(
			clk			=> AD_I_clk,
			reset			=> AD_I_reset,
			rd				=> SA_readFIFO,
			wr				=> SA_writeFIFO,
			w_data		=> SA_ActuationCountFI,
			empty			=> open,
			full			=> open,
			r_data		=>	AD_O_ActuationCount
			);
			
		fifo_SetPoints: entity work.fifo(arch)
		generic map (B => 32, W => FIFO_DEPTH)
		port map(
			clk			=> AD_I_clk,
			reset			=> AD_I_reset,
			rd				=> SA_readFIFO,
			wr				=> SA_writeFIFO,
			w_data		=> SA_ActuationSPFI,
			empty			=> open,
			full			=> open,
			r_data		=>	AD_O_SetPoints

			);
		
	fifo_SP1Time: entity work.fifo(arch)
		generic map (B => 32, W => FIFO_DEPTH)
		port map(
			clk			=> AD_I_clk,
			reset			=> AD_I_reset,
			rd				=> SA_readFIFO,
			wr				=> SA_writeFIFO,
			w_data		=> SA_SP1TimeFI,
			empty			=> open,
			full			=> open,
			r_data		=>	AD_O_SP1Time
			);

	fifo_MinTime: entity work.fifo(arch)
		generic map (B => 32, W => FIFO_DEPTH)
		port map(
			clk			=> AD_I_clk,
			reset			=> AD_I_reset,
			rd				=> SA_readFIFO,
			wr				=> SA_writeFIFO,
			w_data		=>	SA_MinTimeFI,
			empty			=> open,
			full			=> open,
			r_data		=> AD_O_MinTime
			);
			
		fifo_ActTime: entity work.fifo(arch)
		generic map (B => 32, W => FIFO_DEPTH)
		port map(
			clk			=> AD_I_clk,
			reset			=> AD_I_reset,
			rd				=> SA_readFIFO,
			wr				=> SA_writeFIFO,
			w_data		=>	SA_ActTimeFI,
			empty			=> open,
			full			=> open,
			r_data		=> AD_O_ActTime
			);
		
		-- BPS		
		fifo_BPS_Error: entity work.fifo(arch)
		generic map (B => 32, W => FIFO_DEPTH)
		port map(
			clk			=> AD_I_clk,
			reset			=> AD_I_reset,
			rd				=> SA_readFIFO,
			wr				=> SA_writeFIFO,
			w_data		=>	AD_I_BPSErrors,
			empty			=> open,
			full			=> open,
			r_data		=> AD_O_BPSErrors
			);


		fifo_BPSSp1MaxTime: entity work.fifo(arch)
		generic map (B => 32, W => FIFO_DEPTH)
		port map(
			clk			=> AD_I_clk,
			reset			=> AD_I_reset,
			rd				=> SA_readFIFO,
			wr				=> SA_writeFIFO,
			w_data		=>	AD_I_BPSSp1MaxTime,
			empty			=> open,
			full			=> open,
			r_data		=> AD_O_BPSSp1MaxTime
			);
 			
		
		fifo_BPSSp1MinTime: entity work.fifo(arch)
		generic map (B => 32, W => FIFO_DEPTH)
		port map(
			clk			=> AD_I_clk,
			reset			=> AD_I_reset,
			rd				=> SA_readFIFO,
			wr				=> SA_writeFIFO,
			w_data		=>	AD_I_BPSSp1MinTime,
			empty			=> open,
			full			=> open,
			r_data		=> AD_O_BPSSp1MinTime
			);
		
		fifo_BPSMinPosMaxTime: entity work.fifo(arch)
		generic map (B => 32, W => FIFO_DEPTH)
		port map(
			clk			=> AD_I_clk,
			reset			=> AD_I_reset,
			rd				=> SA_readFIFO,
			wr				=> SA_writeFIFO,
			w_data		=>	AD_I_BPSMinPosMaxTime,
			empty			=> open,
			full			=> open,
			r_data		=> AD_O_BPSMinPosMaxTime
			);
			
		fifo_BPSMinPosMinTime: entity work.fifo(arch)
		generic map (B => 32, W => FIFO_DEPTH)
		port map(
			clk			=> AD_I_clk,
			reset			=> AD_I_reset,
			rd				=> SA_readFIFO,
			wr				=> SA_writeFIFO,
			w_data		=>	AD_I_BPSMinPosMinTime,
			empty			=> open,
			full			=> open,
			r_data		=> AD_O_BPSMinPosMinTime
			);		
		fifo_BPSMinPosMaxValue: entity work.fifo(arch)
		generic map (B => 32, W => FIFO_DEPTH)
		port map(
			clk			=> AD_I_clk,
			reset			=> AD_I_reset,
			rd				=> SA_readFIFO,
			wr				=> SA_writeFIFO,
			w_data		=>	AD_I_BPSMinPosMaxValue,
			empty			=> open,
			full			=> open,
			r_data		=> AD_O_BPSMinPosMaxValue
			);
			
		fifo_BPSMinPosMinValue: entity work.fifo(arch)
		generic map (B => 32, W => FIFO_DEPTH)
		port map(
			clk			=> AD_I_clk,
			reset			=> AD_I_reset,
			rd				=> SA_readFIFO,
			wr				=> SA_writeFIFO,
			w_data		=>	AD_I_BPSMinPosMinValue,
			empty			=> open,
			full			=> open,
			r_data		=> AD_O_BPSMinPosMinValue
			);
			
		fifo_BPSActMaxTime: entity work.fifo(arch)
		generic map (B => 32, W => FIFO_DEPTH)
		port map(
			clk			=> AD_I_clk,
			reset			=> AD_I_reset,
			rd				=> SA_readFIFO,
			wr				=> SA_writeFIFO,
			w_data		=>	AD_I_BPSActMaxTime,
			empty			=> open,
			full			=> open,
			r_data		=> AD_O_BPSActMaxTime
			);
			
		fifo_BPSActMinTime: entity work.fifo(arch)
		generic map (B => 32, W => FIFO_DEPTH)
		port map(
			clk			=> AD_I_clk,
			reset			=> AD_I_reset,
			rd				=> SA_readFIFO,
			wr				=> SA_writeFIFO,
			w_data		=>	AD_I_BPSActMinTime,
			empty			=> open,
			full			=> open,
			r_data		=> AD_O_BPSActMinTime
			);
			
		-- SPI
		fifo_CHA_Mon: entity work.fifo(arch)
		generic map (B => 32, W => FIFO_DEPTH)
		port map(
			clk			=> AD_I_clk,
			reset			=> AD_I_reset,
			rd				=> SA_readFIFO,
			wr				=> SA_writeFIFO,
			w_data		=>	SA_ChA_Mon_Now,
			empty			=> open,
			full			=> open,
			r_data		=> AD_O_CHA_Mon
			);
		
		fifo_CHB_Mon: entity work.fifo(arch)
		generic map (B => 32, W => FIFO_DEPTH)
		port map(
			clk			=> AD_I_clk,
			reset			=> AD_I_reset,
			rd				=> SA_readFIFO,
			wr				=> SA_writeFIFO,
			w_data		=>	SA_ChB_Mon_Now,
			empty			=> open,
			full			=> open,
			r_data		=> AD_O_CHB_Mon
			);
	
		fifo_INDX_Mon: entity work.fifo(arch)
		generic map (B => 32, W => FIFO_DEPTH)
		port map(
			clk			=> AD_I_clk,
			reset			=> AD_I_reset,
			rd				=> SA_readFIFO,
			wr				=> SA_writeFIFO,
			w_data		=>	SA_Indx_Mon_Now,
			empty			=> open,
			full			=> open,
			r_data		=> AD_O_INDX_Mon
			);
			
		fifo_Velocity: entity work.fifo(arch)
		generic map (B => 32, W => FIFO_DEPTH)
		port map(
			clk			=> AD_I_clk,
			reset			=> AD_I_reset,
			rd				=> SA_readFIFO,
			wr				=> SA_writeFIFO,
			w_data		=>	SA_Velocity,
			empty			=> open,
			full			=> open,
			r_data		=> AD_O_Velocity
			);
		
		-- Instantiations
		AD_USB_Ack: entity work.edge_detector(v0)
		port map(
			I_rst			=> AD_I_reset,
			I_clk			=> AD_I_clk,
			--
			I_enable		=> '1',
			I_edge_type	=> '1',
			--
			I_data_in	=> AD_I_USBAcknowledge,
			O_edge_out	=> SA_AcknowledgeDG
		);


	-- Assignments
	
	SA_readFIFO <= SA_AcknowledgeDG;
	AD_O_FIFOFull <= SA_FIFOfull;
	AD_O_DataGood 	<= not(SA_FIFOempty);

	SA_ActuationPosFI(31 downto 25)		<= (others => '0');
	SA_ActuationPosFI(24 downto 16) 		<= AD_I_MinDepth;
	SA_ActuationPosFI(15 downto 9) 		<= (others => '0');
	SA_ActuationPosFI(8 downto 0) 		<= AD_I_StartDepth;

	SA_ActuationErrorFI(31 downto 25)	<= (others => '0');
	SA_ActuationErrorFI(24 downto 16) 	<= SA_QuadErrorValueNow;
	SA_ActuationErrorFI(15 downto 8)		<= (others => '0');
	SA_ActuationErrorFI(7)					<= SA_OverCountNow;
	SA_ActuationErrorFI(6)					<= SA_UnderCountNow;
	SA_ActuationErrorFI(5 downto 3)		<= (others => '0');
	SA_ActuationErrorFI(2 downto 0)		<= SA_QuadErrorCodeNow;
	
	SA_ActuationCountFI(31 downto 24) 	<= (others => '0');
	SA_ActuationCountFI(23 downto 0)  	<= AD_I_Count;
	
	SA_ActuationSPFI(31 downto 25)		<= (others => '0');
	SA_ActuationSPFI(24 downto 16)		<= SA_SP2Now;
	SA_ActuationSPFI(15 downto 9)			<= (others => '0');
	SA_ActuationSPFI(8 downto 0)			<= SA_SP1LockedNow;
	
	SA_SP1TimeFI				<= AD_I_SP1Time;
	SA_MinTimeFI				<= AD_I_MinTime;
	SA_ActTimeFI				<= AD_I_ActTime;	
	
	SA_Velocity					<= AD_I_Velocity;
	
	-- Processes
	process(AD_I_clk, AD_I_reset, AD_I_CaptureData, SA_FIFOfull)
		begin		
		if (AD_I_reset = '1') then
								
			SA_QuadErrorValueNow <= (others => '0');
			SA_QuadErrorCodeNow 	<= (others => '0');
				
			SA_UnderCountNow		<= '0';
			SA_OverCountNow		<= '0';
				
			SA_SP1LockedNow		<= (others => '0');
			SA_SP2Now				<= (others => '0');
			
			SA_ChA_Mon_Now 		<= (others => '0');
			SA_ChB_Mon_Now 		<= (others => '0');
			SA_Indx_Mon_Now 		<= (others => '0');
				
		elsif (AD_I_clk'event and AD_I_clk = '1') then
				
			SA_QuadErrorValueNow <= SA_QuadErrorValueNext;
			SA_QuadErrorCodeNow 	<= SA_QuadErrorCodeNext; 
				
			SA_UnderCountNow		<= SA_UnderCountNext;
			SA_OverCountNow 		<= SA_OverCountNext;
				
			SA_SP1LockedNow 		<= SA_SP1LockedNext;
			SA_SP2Now				<= SA_SP2Next;
			
			SA_ChA_Mon_Now 		<= SA_ChA_Mon_Next;
			SA_ChB_Mon_Now 		<= SA_ChB_Mon_Next;
			SA_Indx_Mon_Now 		<= SA_Indx_Mon_Next;
				
			if (AD_I_CaptureData = '1' and SA_FIFOfull = '0') then
				SA_writeFIFO <= '1';
			else
				SA_writeFIFO <= '0';
			end if;
				
		end if;
	end process;
	
	-- Quad Error	
	process(AD_I_QuadFlag, SA_QuadErrorValueNow, SA_QuadErrorCodeNow,
	AD_I_QuadErrorValue, AD_I_QuadErrorCode, AD_I_ActuateTrig)
		begin
	
		SA_QuadErrorValueNext <= SA_QuadErrorValueNow;
		SA_QuadErrorCodeNext <= SA_QuadErrorCodeNow; 
		
		if (AD_I_QuadFlag = '1') then
			SA_QuadErrorValueNext <=AD_I_QuadErrorValue;
			SA_QuadErrorCodeNext <= AD_I_QuadErrorCode;
		--Reset these values at the start of an actuation
		elsif (AD_I_QuadFlag = '0' and AD_I_ActuateTrig = '1') then
			SA_QuadErrorValueNext <= (others => '0');
			SA_QuadErrorCodeNext <= (others => '0');
		end if;
	end process;
	
	-- Over/Under Counts
	process(AD_I_UnderOverTrig, SA_UnderCountNow, SA_OverCountNow,
	AD_I_UnderCapture, AD_I_OverCapture, AD_I_ActuateTrig)
		begin
		
		SA_UnderCountNext <= SA_UnderCountNow;
		SA_OverCountNext <= SA_OverCountNow;
		
		if (AD_I_UnderOverTrig  = '1') then
			SA_UnderCountNext <= AD_I_UnderCapture;
			SA_OverCountNext <= AD_I_OverCapture;
		elsif (AD_I_ActuateTrig = '1') then
		-- This ensures these are reset before every actuation
			SA_UnderCountNext <= '0';
			SA_OverCountNext <= '0';	
		end if;
	end process;
	
	-- Set Points
	process(AD_I_ActuateTrig, SA_SP1LockedNow, SA_SP2Now, AD_I_SP1Locked, AD_I_SP2)
		begin
		
		SA_SP1LockedNext <= SA_SP1LockedNow;
		SA_SP2Next <= SA_SP2Now;
		
		--Grab these values at the start of every actuation
		if (AD_I_ActuateTrig = '1') then
			SA_SP1LockedNext <= AD_I_SP1Locked;
			SA_SP2Next <= AD_I_SP2;
		end if;
	end process;

	-- SPI CHA 
	process(AD_I_CHA_Complete, AD_I_ActuateTrig, AD_I_CHA_Mon, SA_ChA_Mon_Now)
	begin
	
		SA_ChA_Mon_Next <= SA_ChA_Mon_Now;
		-- Assume this occurs before the end of an actuation!
		if (AD_I_CHA_Complete = '1') and (AD_I_ActuateTrig = '0') then
			SA_ChA_Mon_Next(31 downto 16)	<= (others => '0');
			SA_ChA_Mon_Next(15 downto 0)  <= AD_I_CHA_Mon;
		elsif (AD_I_CHA_Complete = '0') and (AD_I_ActuateTrig = '1') then
			-- Resets the value at the start of an actuation
			SA_ChA_Mon_Next(31 downto 16)	<= (others => '0');
			SA_ChA_Mon_Next(15 downto 0)  <= (others => '0');
		end if;
	end process;

	-- SPI CHB 
	process(AD_I_CHB_Complete, AD_I_ActuateTrig, AD_I_CHB_Mon, SA_ChB_Mon_Now)
	begin
	
		SA_ChB_Mon_Next <= SA_ChB_Mon_Now;	
		if (AD_I_CHB_Complete = '1') and (AD_I_ActuateTrig = '0') then
			SA_ChB_Mon_Next(31 downto 16)	<= (others => '0');
			SA_ChB_Mon_Next(15 downto 0)  <= AD_I_CHB_Mon;
		elsif (AD_I_CHB_Complete = '0') and (AD_I_ActuateTrig = '1') then
			-- Resets the value at the start of an actuation
			SA_ChB_Mon_Next(31 downto 16)	<= (others => '0');
			SA_ChB_Mon_Next(15 downto 0)  <= (others => '0');
		end if;
	end process;
	
	-- SPI INDX 
	process(AD_I_Indx_Complete, AD_I_ActuateTrig, AD_I_Indx_Mon, SA_Indx_Mon_Now)
	begin
	
		SA_Indx_Mon_Next <= SA_Indx_Mon_Now;	
		if (AD_I_Indx_Complete = '1') and (AD_I_ActuateTrig = '0') then
			SA_Indx_Mon_Next(31 downto 16)	<= (others => '0');
			SA_Indx_Mon_Next(15 downto 0)  <= AD_I_Indx_Mon;
		elsif (AD_I_Indx_Complete = '0') and (AD_I_ActuateTrig = '1') then
			-- Resets the value at the start of an actuation
			SA_Indx_Mon_Next(31 downto 16)	<= (others => '0');
			SA_Indx_Mon_Next(15 downto 0)  <= (others => '0');
		end if;
	end process;

end arch;

