----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    15:29:56 10/06/2009 
-- Design Name: 
-- Module Name:    Capture - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.numeric_Std.all;


---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity Capture is

    Port ( 
			CAP_I_clk 				: in  STD_LOGIC;
         CAP_I_reset 			: in  STD_LOGIC;
			
			CAP_I_RequiredHold	: in std_logic_vector(5 downto 0);
			CAP_I_CurrentHold		: in std_logic_vector(5 downto 0);
			CAP_I_StartCapture	: in std_logic;
			CAP_I_SettleTime		: in std_logic_vector(31 downto 0);
			CAP_I_KillCapture		: in std_logic;
			CAP_I_WaitTime			: in std_logic_vector(31 downto 0);
			
			CAP_O_HoldState		: out std_logic_vector(5 downto 0);
			CAP_O_CapComplete		: out std_logic;
			CAP_O_Trigger			: out std_logic;
			CAP_O_Count				: out std_logic;
			CAP_O_OverShoot		: out std_logic;
			CAP_O_UnderShoot		: out std_logic;
			CAP_O_Active			: out std_logic;
			CAP_O_AquireActData	: out std_logic;
			CAP_O_UnderOverShoot	: out std_logic;
			
			CAP_O_BoostEnable		: out std_logic
			);
			
			
end Capture;

architecture arch of Capture is

			--constant CAP_SettleTime		: std_logic_vector(31 downto 0) := "00000000000000000000000000010100";--0.5 us
			--constant CAP_SettleTime		: std_logic_vector(31 downto 0) := "00000010011000100101101000000000";--1.0 s
			-- General Signals
			
			type TA_state_type is 	
								(
								State_Initialise,
								State_STart,
								State_Capture1,
								State_Capture2,
								State_Capture3,
								State_Wait1,
								State_Wait2
								);
								
			signal TA_state_reg, TA_state_next: TA_state_type;
			
			signal CAP_HoldStateAbove	: std_logic_vector(5 downto 0);
			signal CAP_HoldStateBelow	: std_logic_vector(5 downto 0);
			signal CAP_HoldState			: std_logic_vector(5 downto 0);
			signal CAP_HoldStateNext	: std_logic_vector(5 downto 0);
			signal CAP_Undershoot		: std_logic;
			signal CAP_OverShoot			: std_logic;
			signal CAP_CorrectPos		: std_logic;
			signal CAPTURE_TIME_NO_CONNECTION : std_logic_vector(31 downto 0);
	
			
			--Timer Signals
			
			signal CAP_TimerStart		: std_logic;
			signal CAP_TimerComplete	: std_logic;
			signal CAP_TimeIn				: std_logic_vector(31 downto 0);
			
begin

			-- Instantiate the Settling Time		
			Settle_Time: entity work.Generic_Timer(arch)
			generic map (N=>32)
			port map (
						clk => CAP_I_clk,
						reset => CAP_I_reset,
						start => CAP_TimerStart,
						en => '1',
						synch_clear => '0',
						time_max_in => CAP_TimeIn,
						time_current_out => CAPTURE_TIME_NO_CONNECTION,
						tick => CAP_TimerComplete
						);
						
			-- Internal Assignments
			
			
			
			-- Assignments to Output Pins (where necessary)

			-- Next State Logic & Data Path Functional Units/Routing
			
			process(CAP_I_clk, CAP_I_reset, CAP_I_KillCapture)
			
			begin
			
				if (CAP_I_reset = '1') then
					TA_state_reg <= State_Initialise;
				elsif (CAP_I_clk'event and CAP_I_clk = '1') then
					if (CAP_I_KillCapture = '1') Then
						TA_state_reg <= State_Initialise;
					else
						TA_state_reg <= TA_state_next;
						CAP_HoldState <= CAP_HoldStateNext;
					end if;
				
					
				end if;
				
			end process;
			
			process(TA_state_reg, CAP_I_StartCapture, CAP_CorrectPos, CAP_HoldState,
			CAP_I_CurrentHold, CAP_TimerComplete, CAP_OverShoot, CAP_UnderShoot,
			CAP_HoldStateBelow, CAP_HoldStateAbove, CAP_I_SettleTime, CAP_I_WaitTime)
	
				begin
				--General Signals
				TA_state_next <= TA_state_reg;
				CAP_HoldStateNext <= Cap_HoldState;
					
				--Defaults
				CAP_O_CapComplete <= '0';
				CAP_O_HoldState <= "000000";
				CAP_TimerStart <='0';
				
				CAP_O_Count	<= '0';			
				CAP_O_OverShoot <= '0';		
				CAP_O_UnderShoot	<= '0';
				CAP_O_Active <= '0';				
				CAP_O_Trigger <= '0';
				CAP_O_AquireActData <= '0';
				CAP_O_UnderOverShoot <= '0';
				
				CAP_O_BoostEnable		<= '0';
			
				CAP_TimeIn <= CAP_I_SettleTime;
	
				case TA_state_reg is
	
				-- System Initialisation on Power Up
				when State_Initialise =>
					CAP_HoldStateNext <= "000000";
					TA_state_next <= State_Start;
					
					
				when State_Start =>
					CAP_HoldStateNext <= CAP_I_CurrentHold;
					Cap_O_HoldState <= "000000";
				if (CAP_I_StartCapture = '1') then
						TA_state_next <= State_Capture1;
						-- Count the Capture
						Cap_O_Trigger <= '1';
						Cap_O_Count <= '1';
					else
						TA_state_next <= State_Start;
					end if;
					
				-- Capture The Target in CapHoldState	
				when State_Capture1 =>
					CAP_O_Active <= '1';
					CAP_O_HoldState <= CAP_HoldState;
					--Trigger The Timer
					CAP_TimerStart <= '1';
					TA_State_Next <= State_Capture2;
					-- Boost the PWM value during the first part of capture.
					CAP_O_BoostEnable <= '1';
					
				when State_Capture2 =>
					--Continue Holding the Current Position
					CAP_O_Active <= '1';
					CAP_O_HoldState <= CAP_HoldState;
					CAP_O_BoostEnable	<= '1';
					if (CAP_TimerComplete = '1') then
						TA_State_Next <= State_Capture3;
					else
						TA_State_Next <= State_Capture2;
					end if;	
					
				
				-- Now Check to see if the hold position needs moving
				when State_Capture3 =>
					--Is the Target In the Correct Position?
					CAP_O_Active <= '1';
					if (CAP_CorrectPos = '1') then
						CAP_O_HoldState <= CAP_HoldState;
						CAP_O_CapComplete <= '0';
						TA_State_Next <= State_Wait1;
						CAP_O_Trigger	<= '0';			
						CAP_O_OverShoot <= '0';		
						CAP_O_UnderShoot	<= '0';
						CAP_TimeIn <= CAP_I_WaitTime;				
					end if;
					
					if (CAP_OverShoot = '1') then
						CAP_O_HoldState <= CAP_HoldState;
						CAP_HoldStateNext <= CAP_HoldStateBelow;
						TA_State_Next <= State_Capture1;
						CAP_O_Trigger	<= '1';			
						CAP_O_OverShoot <= '1';		
						CAP_O_UnderShoot	<= '0';
						CAP_O_UnderOverShoot <= '1';						
					end if;
					
					if (CAP_UnderShoot = '1') then
						CAP_O_HoldState <= CAP_HoldState;
						CAP_HoldStateNext <= CAP_HoldStateAbove;
						TA_State_Next <= State_Capture1;
						CAP_O_Trigger	<= '1';			
						CAP_O_OverShoot <= '0';		
						CAP_O_UnderShoot	<= '1';			
						CAP_O_UnderOverShoot <= '1';						

					end if;
					
				when State_Wait1 =>
					CAP_TimeIn <= CAP_I_WaitTime;
					CAP_O_Active <= '1';
					CAP_TimerStart <= '1';
					CAP_O_AquireActData <= '1';
					CAP_O_HoldState <= CAP_HoldState;
					TA_State_Next<= State_Wait2;				

				
				when State_Wait2 =>
					CAP_TimeIn <= CAP_I_WaitTime;
					CAP_O_Active <= '1';
					CAP_O_HoldState <= CAP_HoldState;
					
					if (CAP_TimerComplete = '1') then
						TA_State_Next <= State_Start;
						CAP_O_CapComplete <= '1';
					else
						TA_State_Next <= State_Wait2;
					end if;	
					
				end case;

			end process;
				
		CAP_HoldStateAbove <= std_logic_vector(unsigned(CAP_I_CurrentHold) + 1);
		CAP_HoldStateBelow <= std_logic_vector(unsigned(CAP_I_CurrentHold) - 1);
		
		process(CAP_I_CurrentHold, CAP_I_RequiredHold)
		
			begin
			
			CAP_UnderShoot <= '0';
			CAP_OverShoot <= '0';
			CAP_CorrectPos <= '0';
			
			if (unsigned(CAP_I_CurrentHold) > unsigned (CAP_I_RequiredHold)) then
				CAP_UnderShoot <= '0';
				CAP_OverShoot <= '1';
				CAP_CorrectPos <= '0';
			end if;
			
			if (unsigned(CAP_I_CurrentHold) < unsigned (CAP_I_RequiredHold)) then
				CAP_UnderShoot <= '1';
				CAP_OverShoot <= '0';
				CAP_CorrectPos <= '0';
			end if;
			
			if (unsigned(CAP_I_CurrentHold) = unsigned(CAP_I_RequiredHold)) then
				CAP_UnderShoot <='0';
				CAP_OverShoot <= '0';
				CAP_CorrectPos <= '1';
			end if;
		
		end process;
				
end arch;

