---------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    14:32:48 10/20/2009 
-- Design Name: 
-- Module Name:    ISIS_Timer_002 - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
--use work.target_types.all;


---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity ISIS_Timer_002 is

	Generic	(
				C_isis_t1_counter_width	: natural := 30;
				C_isis_t2_counter_width	: natural := 20;
				C_legal_spill_width : natural := 20
				--C_isis_mod_count_max		: natural := 64
				);

    port(
			I_rst				: in	std_logic;
			I_40m_clk		: in	std_logic;
			--
			I_delay_t1		: in	std_logic_vector(C_isis_t1_counter_width - 1 downto 0);
			I_delay_t2		: in	std_logic_vector(C_isis_t2_counter_width - 1 downto 0);
			I_LegalSpill	: in	std_logic_vector(C_legal_spill_width - 1 downto 0);
			I_DAQTrigger	: in  std_logic_vector(C_legal_spill_width - 1 downto 0);
			
			I_enableD2		: in  std_logic;
			I_isis_gms		: in	std_logic;
			--I_mod_count	: in	std_logic_vector(5 downto 0);
			--I_mod_count	: in	natural range 0 to C_isis_mod_count_max;
		
			--I_delay_t1	: in	std_logic_vector(20 downto 0);
			--I_delay_t2	: in	std_logic_vector(14 downto 0);
			--I_mod_count	: in	natural range 0 to 64;
			--
		
			--
			O_actuate				: out	std_logic;		
			O_trigAccept			: out std_logic;
			O_trigAccept_NoGate	: out std_logic;
			O_LegalSpill			: out std_logic
			--
			--O_ready		: out std_logic
	);

end ISIS_Timer_002;

architecture arch of ISIS_Timer_002 is	

	-- Constants
	-----------------------------
	
	constant CA_filter_bit_width	: positive range 2 to integer'high := 4;
	
	-- Signals
	-----------------------------
	
	--signal SA_timer_enable	: std_logic;
	
	signal SA_isis_gms_filt						: std_logic;
	signal SA_isis_gms_trig						: std_logic;
	
	--signal SA_isis_gms_t_in	: std_logic;
	signal SA_isis_gms_t1						: std_logic;
	signal ISIS_DELAY_1_NO_CONNECTION		: std_logic;
	signal ISIS_DELAY_2_NO_CONNECTION		: std_logic;
	signal ISIS_LEGAL_SPILL_NO_CONNECTION	: std_logic;
	signal DAQ_TRIGGER_NO_CONNECTION			: std_logic;
	signal ID_Delay2Trigger						: std_logic;
	signal SA_legal_spill						: std_logic;
	signal SA_daq_trigger						: std_logic;
	--signal SA_isis_gms_t2	: std_logic;


begin

	-- Instance port mapping
	-----------------------------
	
	-- Input glitch filter
	U_filter: entity work.glitch_filter
		generic map(
			G_bit_width	=> CA_filter_bit_width
		)
		port map(
			I_rst			=> I_rst,
			I_clk			=> I_40m_clk,
			--
			I_data_in	=> I_isis_gms,
			O_data_out	=> SA_isis_gms_filt
		);
	
	-----------------------------
	
	-- ISIS GMS edge detector
	U_index_edge_detector: entity work.edge_detector
		port map(
			I_rst			=> I_rst,
			I_clk			=> I_40m_clk,
			--
			I_enable		=> '1',
			I_edge_type	=> '1',
			--
			I_data_in	=> SA_isis_gms_filt,
			O_edge_out	=> SA_isis_gms_trig
		);
	
	-----------------------------
	
	-- Timer T1
	U_timer_t1: entity work.trigger_delay
		generic map(
			G_counter_width => C_isis_t1_counter_width
		)
		port map(
			I_rst			=> I_rst,
			I_clk			=> I_40m_clk,
			--
			I_delay		=> I_delay_t1,
			I_trig		=> SA_isis_gms_trig,
			--I_trig		=> SA_isis_gms_t_in,
			--
			O_trig_out	=> SA_isis_gms_t1,
			--
			O_ready		=> ISIS_DELAY_1_NO_CONNECTION
		);
	
	-----------------------------
	
	-- Timer T2
	U_timer_t2: entity work.trigger_delay
		generic map(
			G_counter_width => C_isis_t2_counter_width
		)
		port map(
			I_rst			=> I_rst,
			I_clk			=> I_40m_clk,
			--
			I_delay		=> I_delay_t2,
			I_trig		=> ID_Delay2Trigger,
			--
			O_trig_out	=> O_actuate,
			--
			O_ready		=> ISIS_DELAY_2_NO_CONNECTION
		);
	
	-----------------------------
	
	-- Legal Spill
	U_legal_spill: entity work.trigger_delay
		generic map(
			G_counter_width => C_legal_spill_width
		)
		port map(
			I_rst			=> I_rst,
			I_clk			=> I_40m_clk,
			--
			I_delay		=> I_LegalSpill,
			I_trig		=> SA_isis_gms_trig,
			--
			O_trig_out	=> SA_legal_spill,
			--
			O_ready		=> ISIS_LEGAL_SPILL_NO_CONNECTION
		);
		
	-- DAQ Trigger
	U_daq_trigger: entity work.trigger_delay
		generic map(
			G_counter_width => C_legal_spill_width
		)
		port map(
			I_rst			=> I_rst,
			I_clk			=> I_40m_clk,
			--
			I_delay		=> I_DAQTrigger,
			I_trig		=> SA_isis_gms_trig,
			--
			O_trig_out	=> SA_daq_trigger,
			--
			O_ready		=> DAQ_TRIGGER_NO_CONNECTION	
		);
	
	-- Instantiate Pulse Generator
	TriggerAccept : entity work.FixedWidthPulse(arch)
	
	Port map	( 
				clk 					=> I_40m_clk,			-- : in  STD_LOGIC;
				reset			 		=> I_rst,				-- : in  STD_LOGIC;
				FWP_I_SignalIn		=> ID_Delay2Trigger,	-- : in  std_logic;
				FWP_O_PulseOut		=> O_trigAccept		-- : out std_logic
				);
				
	-- Instantiate Pulse Generator for DAQ Trigger
	TriggerAccept_NoGate : entity work.FixedWidthPulse(arch)
	
	Port map	( 
				clk 					=> I_40m_clk,					-- : in  STD_LOGIC;
				reset			 		=> I_rst,						-- : in  STD_LOGIC;
				FWP_I_SignalIn		=> SA_daq_trigger,			-- : in  std_logic;
				FWP_O_PulseOut		=> O_trigAccept_NoGate		-- : out std_logic
				);
				
				
	-- Instantiate Pulse Generator
	LegalSpill : entity work.FixedWidthPulse2(arch)
	
	Port map	( 
				clk 					=> I_40m_clk,			-- : in  STD_LOGIC;
				reset			 		=> I_rst,				-- : in  STD_LOGIC;
				FWP_I_SignalIn		=> SA_legal_spill,	-- : in  std_logic;
				FWP_O_PulseOut		=> O_LegalSpill		-- : out std_logic
				);			
	
	ID_Delay2Trigger <= (I_enableD2 AND SA_isis_gms_t1);
	

end arch;

