----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    11:01:45 03/09/2010 
-- Design Name: 
-- Module Name:    Notification_Handler - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.numeric_std.all;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity Notification_Handler is
    Port (
			NH_I_clk 					: in  STD_LOGIC;
			NH_I_reset 					: in  STD_LOGIC;
			
			--NH_I_Notify_001			: in std_logic;
			--NH_I_Notify_002			: in std_logic;
			--NH_I_Notify_003			: in std_logic;
			
			NH_I_Notify_001_set		: in std_logic;
			NH_I_Notify_002_set		: in std_logic;
			NH_I_Notify_003_set		: in std_logic;
			
			NH_I_USBAck					: in std_logic;
			
			--NH_O_NotifyReset_001		: out std_logic;
			--NH_O_NotifyReset_002		: out std_logic;
			--NH_O_NotifyReset_003		: out std_logic;

			NH_O_NotifyFlag			: out std_logic;
			NH_O_NotificationCode	: out std_logic_vector(7 downto 0)
			
			);
end Notification_Handler;

architecture arch of Notification_Handler is

	signal SA_USBAck				: std_logic;
	signal SA_readFIFO			: std_logic;
	signal SA_writeFIFO			: std_logic;
	signal SA_writeFIFO_Next	: std_logic;
	signal SA_NotifyFI_Now		: std_logic_vector(7 downto 0);
	signal SA_NotifyFI_Next		: std_logic_vector(7 downto 0);
	signal SA_FIFOempty			: std_logic;
	signal SA_NOTIFY_FIFOfull_NO_CONNECTION			: std_logic;
	
	signal state_AckNot1_Now	: std_logic;
	signal state_AckNot1_Next	: std_logic;
	
	signal state_AckNot2_Now	: std_logic;
	signal state_AckNot2_Next	: std_logic;
	
	signal state_AckNot3_Now	: std_logic;
	signal state_AckNot3_Next	: std_logic;
	
	
	signal SA_SetInitOKNotify			: std_logic;
	signal SA_ResetInitOKNotify		: std_logic;
	signal SA_InitOKNotifyState		: std_logic;
	signal SA_NextInitOKNotifyState	: std_logic;
	
	signal SA_SetParkNotify				: std_logic;
	signal SA_ResetParkNotify			: std_logic;
	signal SA_ParkNotifyState			: std_logic;
	signal SA_NextParkNotifyState		: std_logic;
	
	signal SA_SetFIFONotify				: std_logic;
	signal SA_ResetFIFONotify			: std_logic;
	signal SA_FIFONotifyState			: std_logic;
	signal SA_NextFIFONotifyState		: std_logic;
	
begin

	SA_SetInitOKNotify <= NH_I_Notify_001_set;
	SA_SetParkNotify <= NH_I_Notify_002_set;
	SA_SetFIFONotify <= NH_I_Notify_003_set;

	-- Instantiate EdgeDetector on USB	
	USB_Reset: entity work.edge_detector(v0)
		port map(
			I_rst			=> NH_I_reset,
			I_clk			=> NH_I_clk,
			--
			I_enable		=> '1',
			I_edge_type	=> '1',
			--
			I_data_in	=> NH_I_USBAck,
			O_edge_out	=> SA_USBAck
		);
	
	-- Instantiate FIFO
	fifo_Notify: entity work.fifo(arch)
		generic map (B => 8, W =>3)
		port map(
			clk			=> NH_I_clk,
			reset			=> NH_I_reset,
			rd				=> SA_readFIFO,
			wr				=> SA_writeFIFO,
			w_data		=> SA_NotifyFI_Now,
			empty			=> SA_FIFOempty,
			full			=> SA_NOTIFY_FIFOfull_NO_CONNECTION,
			r_data		=>	NH_O_NotificationCode
			);
	

	-- Assignments
	NH_O_NotifyFlag 	<= not(SA_FIFOempty);
	SA_readFIFO			<= SA_USBAck;
	
	-- process clock
	process(NH_I_clk, NH_I_reset)
	begin
		if (NH_I_reset = '1') then
			state_AckNot1_Now <= '0';
			state_AckNot2_Now <= '0';
			state_AckNot3_Now <= '0';
			SA_NotifyFI_Now <= (others => '0');
			SA_writeFIFO <= '0';
			--Notifications
			SA_InitOKNotifyState <= '0';
			SA_ParkNotifyState 	<= '0';
			SA_FIFONotifyState <= '0';
		elsif (NH_I_clk'event and NH_I_clk = '1') then
			state_AckNot1_Now	<= State_AckNot1_Next;
			state_AckNot2_Now	<= State_AckNot2_Next;
			state_AckNot3_Now	<= State_AckNot3_Next;
			SA_NotifyFI_Now 	<= SA_NotifyFI_Next;
			SA_writeFIFO <= SA_writeFIFO_Next;
			--Notifications
			SA_InitOKNotifyState <= SA_NextInitOKNotifyState;
			SA_ParkNotifyState 	<= SA_NextParkNotifyState;	
			SA_FIFONotifyState <= SA_NextFIFONotifyState;
		end if;
	end process;

	process(State_AckNot1_Now, SA_InitOKNotifyState, NH_I_Notify_001_set,
	State_AckNot2_Now, SA_ParkNotifyState, NH_I_Notify_002_set,
	State_AckNot3_Now, SA_FIFONotifyState, NH_I_Notify_003_set,
	SA_NOTIFY_FIFOfull_NO_CONNECTION, SA_NotifyFI_Now)
	begin

		state_AckNot1_Next	<= state_AckNot1_Now;
		state_AckNot2_Next	<= state_AckNot2_Now;
		state_AckNot3_Next	<= state_AckNot3_Now;
		SA_ResetInitOKNotify	<= '0';
		SA_ResetParkNotify	<= '0';
		SA_ResetFIFONotify	<= '0';
		SA_NotifyFI_Next		<= (Others => '0');

		SA_writeFIFO_Next		<= '0';
				
			If (state_AckNot1_Now = '0' and SA_InitOKNotifyState = '1') Then
			
				SA_NotifyFI_Next <= std_logic_vector(to_unsigned(1,8));
				state_AckNot1_Next <= '1';
				--state_Next <= State_1;
				SA_writeFIFO_Next <= '1';
				
			Elsif (state_AckNot2_Now = '0' and SA_ParkNotifyState = '1') Then
			
				SA_NotifyFI_Next <= std_logic_vector(to_unsigned(2,8));
				state_AckNot2_Next <= '1';
				--state_Next <= State_2;
				SA_writeFIFO_Next <= '1';

			Elsif (state_AckNot3_Now = '0' and SA_FIFONotifyState = '1') Then
			
				SA_NotifyFI_Next <= std_logic_vector(to_unsigned(3,8));
				state_AckNot3_Next <= '1';
				--state_Next <= State_2;
				SA_writeFIFO_Next <= '1';
				
			Else
				SA_NotifyFI_Next <= std_logic_vector(to_unsigned(0,8));			
			End If;
			
		
			If (NH_I_Notify_001_set = '0' and SA_InitOKNotifyState = '1') Then
				SA_ResetInitOKNotify	<= '1';
				state_AckNot1_Next 	<= '0';
			End If;
			
			If (NH_I_Notify_002_set = '0' and SA_ParkNotifyState = '1') Then
				SA_ResetParkNotify	<= '1';
				state_AckNot2_Next <= '0';
			End If;
			
			If (NH_I_Notify_003_set = '0' and SA_FIFONotifyState = '1') Then
				SA_ResetFIFONotify	<= '1';
				state_AckNot3_Next 	<= '0';
			End If;
			
	End process;
	
	-- System Init OK
	process(SA_SetInitOKNotify, SA_ResetInitOKNotify, SA_InitOKNotifyState)
	begin	
		SA_NextInitOKNotifyState <= SA_InitOKNotifyState;
	
		If (SA_SetInitOKNotify = '1' and SA_ResetInitOKNotify = '0') Then
				SA_NextInitOKNotifyState <= '1';
		Elsif (SA_SetInitOKNotify = '0' and SA_ResetInitOKNotify = '1') Then
				SA_NextInitOKNotifyState <= '0';
		End If;
	End Process;
	
	-- Park Error (Try to Park with Frame NOT raised)
	process(SA_SetParkNotify, SA_ResetParkNotify, SA_ParkNotifyState)
	begin	
		SA_NextParkNotifyState <= SA_ParkNotifyState;
	
		If (SA_SetParkNotify = '1' and SA_ResetParkNotify = '0') Then
				SA_NextParkNotifyState <= '1';
		Elsif (SA_SetParkNotify = '0' and SA_ResetParkNotify = '1') Then
				SA_NextParkNotifyState <= '0';
		End If;
	End Process;
	
	
	-- Actuation Data FIFO Overflow
	process(SA_SetFIFONotify, SA_ResetFIFONotify, SA_FIFONotifyState)
	begin	
		SA_NextFIFONotifyState <= SA_FIFONotifyState;
	
		If (SA_SetFIFONotify = '1' and SA_ResetFIFONotify = '0') Then
				SA_NextFIFONotifyState <= '1';
		Elsif (SA_SetFIFONotify = '0' and SA_ResetFIFONotify = '1') Then
				SA_NextFIFONotifyState <= '0';
		End If;
	End Process;
			
end arch;

