----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    15:28:30 10/21/2009 
-- Design Name: 
-- Module Name:    ConfigurationBlock - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity OutputBlock is

    Port (
			 clk 									: in 	std_logic;			
			 reset 								: in 	std_logic;	
	 
			 OB_I_Actuate						: in 	std_logic;			 
			 OB_O_Actuate1						: out std_logic;
			 OB_O_Actuate2						: out std_logic;
			 OB_O_Actuate3						: out std_logic;
			 OB_O_ActuatingLED				: out std_logic;
			 
			 OB_I_ActuateTrig					: in std_logic;
			 OB_O_ActuateTrig1				: out std_logic;
			 OB_O_ActuateTrig2				: out std_logic;
			 
			 OB_I_MosfetLED					: in std_logic_vector(6 downto 1);
			 OB_O_MosfetLED					: out std_logic_vector(6 downto 1);
			 
			 OB_I_Mosfet						: in std_logic_vector(6 downto 1);
			 OB_O_Mosfet						: out std_logic_vector(6 downto 1);

			 OB_I_HoldLED						: in std_logic;
			 OB_O_HoldLED						: out std_logic;
			 
			 OB_I_ParkLED						: in std_logic;
			 OB_O_ParkLED						: out std_logic;
			 
			 OB_I_Position						: in std_logic_vector(8 downto 0);
			 OB_O_Position						: out std_logic_vector(8 downto 0);
			 			 
			 OB_I_KeyEnableDeGlitch			: in std_logic;
			 OB_O_KeyEnabledDGLED			: out std_logic;
			 
			 OB_I_ActEnabledLED				: in std_logic;
			 OB_O_ActEnabledLED				: out std_logic;
			 
			 OB_I_TargetReady					: in std_logic;
			 OB_O_TargetReady50R				: out std_logic;
			 OB_O_TargetReadyLED				: out std_logic;
			 
			 OB_I_TargetFrameEnable			: in std_logic;
			 OB_O_TargetFrameEnable			: out std_logic;
			 OB_O_FrameEnableRelay			: out std_logic;
			 
			 OB_I_TriggerAccept				: in std_logic;
			 OB_O_TriggerAccept				: out std_logic;
			 
			 OB_I_SpillMarker					: in std_logic;
			 OB_O_SpillMarker					: out std_logic;
			 
			 OB_I_FrameRaised					: in std_logic;
			 OB_O_FrameRaisedLED				: out std_logic;
			 OB_I_FrameLowered				: in std_logic;
			 OB_O_FrameLoweredLED			: out std_logic;
			 
			 OB_I_ErrorLED						: in std_logic;
			 OB_O_ErrorLED						: out std_logic;
			 
			 OB_I_BPS							: in std_logic;
			 OB_I_BPS_control					: in std_logic_vector(2 downto 0);
			 OB_O_BPS_50R						: out std_logic;
			 OB_O_BPS_Relay1					: out std_logic;
			 OB_O_BPS_Relay2					: out std_logic;
			 OB_O_BPS_Relay3					: out std_logic;
			 
			 OB_I_Engineering					: in std_logic;
			 OB_I_1MHZ							: in std_logic;
			 
			 OB_I_EngLampTestLED				: in std_logic;							--	: in std_logic;
			 OB_I_EngLampTestLED_d			: in std_logic_vector(11 downto 0);	--: in std_logic_vector(8 downto 0);
			
			 OB_I_EngLampTestIGBT			: in std_logic;
			 OB_I_EngLampTestIGBT_d			: in std_logic_vector(5 downto 0);
			 
			 OB_I_EngTrigAccept				: in std_logic;
			 OB_I_EngTargetReady				: in std_logic;
			 OB_I_EngFrameEnable				: in std_logic;
			 OB_I_EngActTrig					: in std_logic;
			 OB_I_EngBPS50R					: in std_logic;
			 OB_I_EngActuating				: in std_logic;		
			 OB_I_EngSpillMarker				: in std_logic;
			 OB_I_EngBPSRelays				: in std_logic;
			 OB_I_EngFERelay					: in std_logic;
			 OB_I_EngNudge						: in std_logic;
			 OB_I_EngNudgeControl			: in std_logic_vector(6 downto 1);
			 OB_I_FR_BPS_ON_OFF				: in std_logic;
			 OB_I_BPS_LATCH_ENABLE			: in std_logic;
			 
			 OB_O_CHA_LED						: out std_logic;		
			 OB_I_CHA_LED						: in  std_logic;
			 OB_O_CHB_LED						: out std_logic;		
			 OB_I_CHB_LED						: in  std_logic;
			 OB_O_INDX_LED						: out std_logic;
			 OB_I_INDX_LED						: in  std_logic
			 );
			 
end OutputBlock;

architecture arch of OutputBlock is

	signal sl_index_timer_complete		: std_logic;
	signal sl_index_tick						: std_logic;
	signal sl_index_LED_now					: std_logic;
	signal sl_index_LED_next				: std_logic;

begin 

	-- Instantiate a "x" ms Timer for INDEX		
	INDEX_Timer: entity work.Generic_Timer(arch)
	generic map (N=>19)
	port map   (
		clk => 	clk,
		reset => reset ,
		start => sl_index_tick,
		en => '1',
		synch_clear => '0',
		time_max_in => "1100001101010000000", -- 10ms
		time_current_out => open,
		tick => sl_index_timer_complete
		);
		
	INDEX_tick: entity work.edge_detector(v0)
		port map(
			I_rst			=> reset,
			I_clk			=> clk,
			--
			I_enable		=> '1',
			I_edge_type	=> '1',
			--
			I_data_in	=> OB_I_INDX_LED,
			O_edge_out	=> sl_index_tick
		);

	
	-- CLOCKING PROCESS 
	process(clk, reset)
	begin	
		if (reset = '1') then
			sl_Index_LED_now <= '0';
		elsif (clk'event and clk = '1') then
			sl_Index_LED_now <= sl_Index_LED_next;
		end if;
	end process;
	
	-- Index pulse extender
	process(sl_index_tick, sl_index_timer_complete, sl_Index_LED_now)
	--process
	begin
		sl_Index_LED_next <= sl_Index_LED_now;
	
		if (sl_index_tick = '1' and sl_index_timer_complete = '0') then
			sl_Index_LED_next <= '1';
		elsif (sl_index_timer_complete = '1') then
			sl_Index_LED_next <= '0';
		end if;
	end process;
	
	-- DISPLAY LEDs
	process(OB_I_HoldLED, OB_I_ParkLED, OB_I_KeyEnableDeGlitch,
	OB_I_ActEnabledLED, OB_I_TargetReady, OB_I_Actuate, OB_I_FrameRaised, 
	OB_I_FrameLowered, OB_I_ErrorLED, 
	OB_I_Engineering, OB_I_EngLampTestLED, OB_I_EngLampTestLED_d,
	OB_I_CHA_LED, OB_I_CHB_LED, sl_Index_LED_now)
	
	begin
	
	OB_O_HoldLED			<= '0';
	OB_O_ParkLED			<= '0';
		
	OB_O_KeyEnabledDGLED <= '0';
	OB_O_ActEnabledLED 	<= '0';
	OB_O_TargetReadyLED  <= '0';
	OB_O_ActuatingLED 	<= '0';
			
	OB_O_FrameRaisedLED 	<= '0';
	OB_O_FrameLoweredLED <= '0';
	OB_O_ErrorLED			<= '0';
	
	OB_O_CHA_LED			<= '0';
	OB_O_CHB_LED			<= '0';
	OB_O_INDX_LED			<= '0';
	
		
	if (OB_I_Engineering = '1'  and OB_I_EngLampTestLED = '1') then
	
		OB_O_HoldLED 			<= OB_I_EngLampTestLED_d(1);
		OB_O_ParkLED 			<= OB_I_EngLampTestLED_d(0);
		
		OB_O_KeyEnabledDGLED <= OB_I_EngLampTestLED_d(2);
		OB_O_ActEnabledLED 	<= OB_I_EngLampTestLED_d(3);
		OB_O_TargetReadyLED  <= OB_I_EngLampTestLED_d(4);
		OB_O_ActuatingLED 	<= OB_I_EngLampTestLED_d(5);
			
		OB_O_FrameRaisedLED 	<= OB_I_EngLampTestLED_d(6);
		OB_O_FrameLoweredLED <= OB_I_EngLampTestLED_d(7);
		OB_O_ErrorLED			<= OB_I_EngLampTestLED_d(8);
		
		OB_O_CHA_LED			<= OB_I_EngLampTestLED_d(9);
		OB_O_CHB_LED			<= OB_I_EngLampTestLED_d(10);
		OB_O_INDX_LED			<= OB_I_EngLampTestLED_d(11);
		
	elsif (OB_I_Engineering = '0') then
	
		OB_O_HoldLED 			<= OB_I_HoldLED;
		OB_O_ParkLED 			<= OB_I_ParkLED;
		
		OB_O_KeyEnabledDGLED <= OB_I_KeyEnableDeGlitch;
		OB_O_ActEnabledLED 	<= OB_I_ActEnabledLED;
		OB_O_TargetReadyLED  <= OB_I_TargetReady;
		OB_O_ActuatingLED 	<= OB_I_Actuate;
			
		OB_O_FrameRaisedLED 	<= OB_I_FrameRaised;
		OB_O_FrameLoweredLED <= OB_I_FrameLowered;
		OB_O_ErrorLED			<= OB_I_ErrorLED;
		
		OB_O_CHA_LED			<= OB_I_CHA_LED;
		OB_O_CHB_LED			<= OB_I_CHB_LED;
		OB_O_INDX_LED			<= sl_Index_LED_now;
	
	end if;
	end process;
	
	-- MOSFET LEDs & OCs
	process(OB_I_MosfetLED, OB_I_Mosfet,
	OB_I_Engineering, OB_I_EngLampTestIGBT, OB_I_EngLampTestIGBT_d,
	OB_I_EngNudgeControl, OB_I_EngNudge)
	
	begin
	
	OB_O_MosfetLED 		<= (others => '0');
	OB_O_Mosfet 			<= (others => '0');

	if (OB_I_Engineering = '0') then
		OB_O_MosfetLED 		<= OB_I_MosfetLED;
		OB_O_Mosfet 			<= OB_I_Mosfet;
		
	elsif (OB_I_EngLampTestIGBT = '1' and OB_I_EngNudge = '0') then
		OB_O_MosfetLED 		<= OB_I_EngLampTestIGBT_d;
		OB_O_Mosfet 			<= OB_I_EngLampTestIGBT_d;
		
	elsif (OB_I_EngLampTestIGBT = '0' and OB_I_EngNudge = '1') then
		OB_O_MosfetLED 		<= OB_I_EngNudgeControl;
		OB_O_Mosfet 			<= OB_I_EngNudgeControl;
		
	end if;
	end process;
	
	-- TRIGGER ACCEPT
	process(OB_I_Engineering, OB_I_TriggerAccept, OB_I_1MHZ,
		OB_I_EngTrigAccept)
	
	begin
	
	OB_O_TriggerAccept <= '0';
	
	
	if (OB_I_Engineering = '0') then
		OB_O_TriggerAccept <= OB_I_TriggerAccept;
		--OB_O_TriggerAccept <= '1';
		
	elsif (OB_I_EngTrigAccept = '1') then
		OB_O_TriggerAccept <= OB_I_1MHZ;
		
	end if;
	end process;
	
	-- TARGET READY 50R
	process(OB_I_Engineering, OB_I_TargetReady, OB_I_1MHZ,
		OB_I_EngTargetReady)
	
	begin
	
	OB_O_TargetReady50R <= '0';
	
	if (OB_I_Engineering = '0') then
		OB_O_TargetReady50R		<= OB_I_TargetReady;
		
	elsif (OB_I_EngTargetReady = '1') then
		OB_O_TargetReady50R <= OB_I_1MHZ;
		
	end if;
	end process;
	
	
	-- TARGET FRAME ENABLE 50R
	process(OB_I_Engineering, OB_I_TargetFrameEnable, OB_I_1MHZ,
		OB_I_EngFrameEnable)
	
	begin
	
	OB_O_TargetFrameEnable <= '0';
	
	if (OB_I_Engineering = '0') then
		OB_O_TargetFrameEnable	<= OB_I_TargetFrameEnable;
		
	elsif (OB_I_EngFrameEnable = '1') then
		OB_O_TargetFrameEnable <= OB_I_1MHZ;
		
	end if;
	end process;

	-- ACTUATE TRIGGER
	process(OB_I_Engineering, OB_I_ActuateTrig, OB_I_1MHZ,
		OB_I_EngActTrig)
	
	begin
	
	OB_O_ActuateTrig1 	<= '0';
	OB_O_ActuateTrig2		<= '0';
	
	if (OB_I_Engineering = '0') then
		OB_O_ActuateTrig1 	<= OB_I_ActuateTrig;
		OB_O_ActuateTrig2		<= OB_I_ActuateTrig;
		
	elsif (OB_I_EngActTrig = '1') then
		OB_O_ActuateTrig1 	<= OB_I_1MHZ;
		OB_O_ActuateTrig2		<= OB_I_1MHZ;
		
	end if;
	end process;

	-- BPS 50R
	process( OB_I_BPS, OB_I_FR_BPS_ON_OFF, OB_I_Engineering, OB_I_EngBPS50R, OB_I_1MHZ,
		OB_I_BPS_LATCH_ENABLE, OB_I_FrameRaised)
	
	begin
	
	OB_O_BPS_50R <= '1';
	
	if (OB_I_Engineering = '0') then
		-- If OB_I_BPS_LATCH_ENABLE = 0 then the BPS relay outputs remain at '1' 
		-- (i.e. BPS disabled) when the frame is raised.
		-- Note that OB_I_BPS_LATCH_ENABLE is a latch that will only be set to '1' if 
		-- there is a BPS error when the frame is NOT raised. 
		-- In this case (if the latch itself is operational - see latch code in controller_phaseX.vhd)
		-- then the BPS relays will follow the status of the BPS_state.
		
		if (OB_I_FR_BPS_ON_OFF = '1') then
			OB_O_BPS_50R <= OB_I_BPS;
		else
			if (OB_I_FrameRaised = '1' and OB_I_BPS_LATCH_ENABLE = '0') then
				OB_O_BPS_50R <= '1';
			else
				OB_O_BPS_50R <= OB_I_BPS;
			end if;
		end if;
	else
		if (OB_I_EngBPS50R = '1') then
			OB_O_BPS_50R <= OB_I_1MHZ;
		end if;
	end if;
	end process;

	-- BPS Relay	
	process( OB_I_Engineering, OB_I_BPS, OB_I_BPS_control, OB_I_EngBPSRelays,
		OB_I_BPS_LATCH_ENABLE, OB_I_FR_BPS_ON_OFF, OB_I_FrameRaised)
			
	begin
	
	OB_O_BPS_Relay1 <= '1';
	OB_O_BPS_Relay2 <= '1';
	OB_O_BPS_Relay3 <= '1';
	
	if (OB_I_Engineering = '0') then
		-- If OB_I_BPS_LATCH_ENABLE = 0 then the BPS relay outputs remain at '1' 
		-- (i.e. BPS disabled) when the frame is raised.
		-- Note that OB_I_BPS_LATCH_ENABLE is a latch that will only be set to '1' if 
		-- there is a BPS error when the frame is NOT raised. 
		-- In this case (if the latch itself is operational - see latch code in controller_phaseX.vhd)
		-- then the BPS relays will follow the status of the BPS_state.
		
		if (OB_I_FR_BPS_ON_OFF = '1') then
			OB_O_BPS_Relay1		<=  OB_I_BPS;		
			OB_O_BPS_Relay2		<=  OB_I_BPS;			
			OB_O_BPS_Relay3		<=  OB_I_BPS;	
		else
			if (OB_I_FrameRaised = '1' and OB_I_BPS_LATCH_ENABLE = '0') then
				OB_O_BPS_Relay1 <= '1';
				OB_O_BPS_Relay2 <= '1';
				OB_O_BPS_Relay3 <= '1';
			else
				OB_O_BPS_Relay1		<=  OB_I_BPS;		
				OB_O_BPS_Relay2		<=  OB_I_BPS;			
				OB_O_BPS_Relay3		<=  OB_I_BPS;	
			end if;
		end if;
	else
		if (OB_I_EngBPSRelays = '1') then
			OB_O_BPS_Relay1		<= OB_I_BPS_control(0);
			OB_O_BPS_Relay2		<= OB_I_BPS_control(1);			
			OB_O_BPS_Relay3		<= OB_I_BPS_control(2);
		end if;
	end if;
	end process;
	
	-- Actuating
	process(OB_I_Engineering, OB_I_Actuate, OB_I_1MHZ,
		OB_I_EngActuating)
			
	begin
	
	OB_O_Actuate1 <= '0';
	OB_O_Actuate2 <= '0';
	OB_O_Actuate3 <= '0';
	
	if (OB_I_Engineering = '0') then
		OB_O_Actuate1 <= OB_I_Actuate;
		OB_O_Actuate2 <= OB_I_Actuate;
		OB_O_Actuate3 <= OB_I_Actuate;
	elsif (OB_I_EngActuating = '1') then
		OB_O_Actuate1 <= OB_I_1MHZ;
		OB_O_Actuate2 <= OB_I_1MHZ;
		OB_O_Actuate3 <= OB_I_1MHZ;
	end if;
	end process;
	
	-- Spill Marker
	process(OB_I_Engineering, OB_I_SpillMarker, OB_I_1MHZ,
		OB_I_EngSpillMarker)
	
	begin
	
	OB_O_SpillMarker <= '0';
	
	if (OB_I_Engineering = '0') then
		OB_O_SpillMarker	<= OB_I_SpillMarker;
		
	elsif (OB_I_EngSpillMarker = '1') then
		OB_O_SpillMarker <= OB_I_1MHZ;
		
	end if;
	end process;
	
	-- Frame Enable Relay
	process(OB_I_Engineering, OB_I_TargetFrameEnable,
		OB_I_EngFERelay)
	
	begin
	
	OB_O_FrameEnableRelay <= '0';
	
	if (OB_I_Engineering = '0') then
		OB_O_FrameEnableRelay	<= OB_I_TargetFrameEnable;
		
	elsif (OB_I_EngFERelay = '1') then
		OB_O_FrameEnableRelay <= '1';
		
	end if;
	end process;
	
	OB_O_Position 			<= OB_I_Position;						
		
			
end arch;

