-----------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
--use work.target_types.all;
-----------------------------------------------------------


-- Entity
-----------------------------------------------------------
entity quadrature_decoder is
	generic	(
				G_position_min	: integer := 0;
				G_position_max	: integer := 511
				);
				
	port	(
			I_rst				: in	std_logic;
			I_clk				: in	std_logic;
			I_RstToStart	: in  std_logic;
			--
			I_rst_position	: in	std_logic_vector (8 downto 0);
			I_index_offset	: in	std_logic_vector (8 downto 0);
			--I_reset_error	: in  std_logic;
			--
			I_ch_a			: in	std_logic;
			I_ch_b			: in	std_logic;
			I_index			: in	std_logic;
			--
			O_position		: out	std_logic_vector(8 downto 0);
			O_direction		: out	std_logic;
			--
			O_ErrorFlag		: out std_logic;
			O_error			: out	std_logic_vector(2 downto 0);
			O_error_value	: out	std_logic_vector(8 downto 0)
			);
			
end entity quadrature_decoder;

-- Architecture
-----------------------------------------------------------
architecture v0 of quadrature_decoder is
	
	-- Values / Signals
	-----------------------------
	
	signal SA_count_enable		: std_logic;
	signal SA_count_direction	: std_logic;
	signal SA_count_int			: unsigned(8 downto 0);
	
	signal SA_ch_a_delayed		: std_logic_vector(2 downto 0);
	signal SA_ch_b_delayed		: std_logic_vector(2 downto 0);
	
begin
	
	-- Processes
	-----------------------------
	
	quad_delay: process(I_rst, I_clk)
	begin
		
		if(I_rst = '1') then
			
			SA_ch_a_delayed <= "000";
			SA_ch_b_delayed <= "000";
			
		elsif(I_clk'event and I_clk = '1') then
			
			SA_ch_a_delayed <= SA_ch_a_delayed(1 downto 0) & I_ch_a;
			SA_ch_b_delayed <= SA_ch_b_delayed(1 downto 0) & I_ch_b;
			
		end if;
		
	end process quad_delay;
	
	-----------------------------
	
	quad_count: process(I_rst, I_clk, I_RstToStart, I_rst_position)
	begin
				
		if(I_rst = '1') then
			
			SA_count_int <= (others => '0');
			
			O_direction <= '0';
			O_error <= "000"; --quad_no_error
			O_error_value	<= (others => '0');
			O_ErrorFlag <= '0';
			
		elsif (I_RstToStart = '1') then
		
			SA_count_int <= unsigned(I_rst_position);			
			O_direction <= '0';
			O_error <= "000"; --quad_no_error
			O_error_value	<= (others => '0');
			O_ErrorFlag <= '0';
				
		elsif(I_clk'event and I_clk = '1') then
			
			if(I_index = '1') then
				
				if(SA_count_int /= unsigned(I_index_offset)) then
					O_error <= "100"; --quad_index_error
					O_error_value <= std_logic_vector(SA_count_int);
					O_ErrorFlag <= '1';			
				else 
					O_error <= "000"; --quad_index_error
					O_error_value <= (others => '0');
					O_ErrorFlag <= '0';	
				end if;
				
				SA_count_int <= unsigned(I_index_offset);
				
			elsif(SA_count_enable = '1') then
				
				O_direction <= SA_count_direction;
				
				if(SA_count_direction = '1') then
					
					if(SA_count_int = G_position_max) then
						O_error <= "001"; --quad_max_error
						O_error_value <= std_logic_vector(SA_count_int);
						O_ErrorFlag <= '1';
						SA_count_int <= SA_count_int;
					else
						O_error <= "000"; --quad_index_error
						O_error_value <= (others => '0');
						O_ErrorFlag <= '0';	
						SA_count_int <= SA_count_int + 1;
					end if;
					
				elsif(SA_count_direction = '0') then
					
					if(SA_count_int = G_position_min) then
						O_error <= "010"; --quad_min_error
						O_error_value <= std_logic_vector(SA_count_int);
						O_ErrorFlag <= '1';
						SA_count_int <= SA_count_int;
					else
						O_error <= "000"; --quad_index_error
						O_error_value <= (others => '0');
						O_ErrorFlag <= '0';	
						SA_count_int <= SA_count_int - 1;
					end if;
					
				end if;
				
			end if;
			
			--If (I_reset_error = '1') Then
			--	O_error <= "000"; --quad_no_error
			--	O_error_value	<= (others => '0');
			--	O_ErrorFlag <= '0';
			--End If;
			
		end if;
		
	end process quad_count;
	
	-- Concurrent signal
	-- assignment
	-----------------------------
	
	SA_count_enable <= SA_ch_a_delayed(1) xor SA_ch_a_delayed(2) xor SA_ch_b_delayed(1) xor SA_ch_b_delayed(2);
	SA_count_direction <= SA_ch_a_delayed(1) xor SA_ch_b_delayed(2);
	
	-----------------------------
	
	O_position <= std_logic_vector(SA_count_int);
	
end architecture v0;
