----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    13:27:41 03/21/2012 
-- Design Name: 
-- Module Name:    target_velocity - arch 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity target_velocity is
    Port ( 
	 
				reset 			: in  std_logic;
				clk 				: in  std_logic;
				
				I_Actuating		: in 	std_logic;
				
				I_Locked_SP1			: in 	std_logic_vector(8 downto 0);
				I_Position				: in  std_logic_vector(8 downto 0);
				I_Quadrant				: in 	std_logic_vector(2 downto 0);			
				I_Actuation_timer		: in 	std_logic_vector(31 downto 0);
				
				O_Velocity				: out	std_logic_vector(31 downto 0)
				);
				
end target_velocity;

architecture arch of target_velocity is

	type TA_state_type is 	(
										State_Wait,
										State_1,
										State_2,
										State_3,
										State_4
									);
							
	signal TA_state_now, TA_state_next: TA_state_type;
	
	signal u_upper_point		: unsigned(8 downto 0);
	signal u_lower_point		: unsigned(8 downto 0);
	signal u_locked_SP1		: unsigned(8 downto 0);
	
	signal u_actuation_time	: unsigned(31 downto 0);
	signal u_velocity_now	: unsigned(31 downto 0);
	signal u_velocity_next	: unsigned(31 downto 0);
	signal u_position			: unsigned(8 downto 0);
	
	signal u_difference		: unsigned(31 downto 0);
	
	signal u_upper_point_time_now : unsigned(31 downto 0);
	signal u_upper_point_time_next : unsigned(31 downto 0);
	signal u_lower_point_time_now : unsigned(31 downto 0);
	signal u_lower_point_time_next : unsigned(31 downto 0);

begin

	u_locked_SP1 	<= unsigned(I_Locked_SP1);
	u_position 		<= unsigned(I_Position);

	u_upper_point 	<= (u_locked_SP1 + 9);
	u_lower_point 	<= (u_locked_SP1 + 5);
	
	u_actuation_time <= unsigned(I_Actuation_timer);
	
	u_difference <= (u_lower_point_time_now -  u_upper_point_time_now);
	O_Velocity <= std_logic_vector(u_velocity_now);

	-- Processes
	process(clk, reset)
			
	begin
			
		if (reset = '1') then
			TA_state_now <= State_Wait;
			u_velocity_now		<= (others => '0');
			u_upper_point_time_now	<= (others => '0');
			u_lower_point_time_now	<= (others => '0');
		elsif (clk'event and clk = '1') then
			TA_state_now <= TA_state_next;
			u_velocity_now		<= u_velocity_next;
			u_upper_point_time_now	<= u_upper_point_time_next;
			u_lower_point_time_now	<= u_lower_point_time_next;
		end if;
				
	end process;
	
	process(TA_state_now, TA_state_next, I_Actuating, I_Quadrant,
		u_velocity_now, u_upper_point_time_now, u_lower_point_time_now,
		u_position, u_upper_point, u_actuation_time, u_lower_point, u_difference)
	
	begin
	
		TA_state_next 		<= TA_state_now;
		u_velocity_next	<= u_velocity_now;
		u_upper_point_time_next	<= u_upper_point_time_now;
		u_lower_point_time_next	<= u_lower_point_time_now;
		
		case TA_state_now is
		
			when State_Wait =>
				if (I_Actuating =  '1' and I_Quadrant = "001") then 
					TA_state_next <= State_1;
				end if;
				
			when State_1 =>	
				TA_state_next <= State_2;
				u_velocity_next <= (others => '0');
				u_upper_point_time_next <= (others => '0');
				u_lower_point_time_next	<= (others => '0');
				
			when State_2 =>
				if (I_Actuating = '1') then
					if (u_position = u_upper_point) then
						u_upper_point_time_next <= u_actuation_time;
						TA_state_next <= State_3;
					end if;
				else
					TA_state_next <= State_Wait;
					u_velocity_next <= (others => '0');
					u_upper_point_time_next <= (others => '0');
					u_lower_point_time_next	<= (others => '0');
				end if;
				
			when State_3 =>
				if (I_Actuating = '1') then
					if (u_position = u_lower_point) then
						u_lower_point_time_next <= u_actuation_time;
						TA_state_next <= State_4;
					end if;
				else
					TA_state_next <= State_Wait;
					u_velocity_next <= (others => '0');
					u_upper_point_time_next <= (others => '0');
					u_lower_point_time_next	<= (others => '0');
				end if;
				
			when State_4 =>
				TA_state_next <= State_4;
				u_velocity_next <= u_difference;
				if (I_Actuating = '0') then
					TA_state_next <= State_Wait;
				end if;
				
		end case;
	end process;

end arch;

