----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    10:03:37 07/13/2009 
-- Design Name: 
-- Module Name:    WD_Timer - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.numeric_std.all;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;


-- WDT_O_Out is always high
-- Only when trigger is taken high does the WDT start counting
-- If trigger is taken low the WDT stops counting
-- If the counter reaches a predetermined value THEN the output goes low.
-- This Then has to be reset with WDT

entity WD_Timer is

	generic	(
				N: integer := 24
				);

    Port 	( 
				WDT_I_clk 			: in  STD_LOGIC;
				WDT_I_reset 		: in  STD_LOGIC;
				WDT_I_trigger 		: in  std_logic;
				WDT_I_wdt_reset	: in  std_logic;
				WDT_I_timer_value	: in	std_logic_vector(N-1 downto 0);
				
				WDT_O_out		 	: out std_logic
				);
				
end WD_Timer;

architecture arch of WD_Timer is

	--constant WD_TIME: integer:=99;


	--signal SA_time_next: unsigned(7 downto 0);
	signal SA_timer_start: std_logic;
	signal SA_timer_complete: std_logic;
	signal WDT_TIME_OUT_NO_CONNECTION: std_logic_vector(N-1 downto 0);
	


	type TA_state_type is 	(
									State_Initialise,
									State_Wait_1, State_Trigger,
									State_Wait_2, --State_Decrement,
									State_Time_Out
									);

	signal TA_state_current, TA_state_next: TA_state_type;

begin

	-- Instantiate a N bit timer
	watchdog_timer: entity work.Generic_Timer(arch)
	generic map (N=>N)
	port map   (
					clk => WDT_I_clk,
					reset => WDT_I_reset,
					start => SA_timer_start,
					en => '1',
					synch_clear => '0',
					time_max_in => WDT_I_timer_value,
					time_current_out => WDT_TIME_OUT_NO_CONNECTION,
					tick => SA_timer_complete
					);
					
	
	-- Processes
	process(WDT_I_clk, WDT_I_reset)
		begin
		if WDT_I_reset = '1' then
			TA_state_current <= State_Initialise;
			--SA_time_current <=(others => '0');
		elsif (WDT_I_clk'event and WDT_I_clk = '1') then
			TA_state_current <= TA_state_next;
			--SA_time_current <= SA_time_next;
		end if;
	end process;
	
	
	
	-- FSM
	process(TA_state_Current, WDT_I_Trigger, SA_timer_start,
				SA_timer_complete, WDT_I_wdt_reset)--,SA_time_current, WDT_I_wdt_reset)
	begin
	
		--General Signals
		
		TA_state_next <= TA_state_current;
		WDT_O_out <= '0';
		--SA_time_next <= SA_time_current;
		SA_timer_start <= '0';
		
		case TA_state_current is
			
			when State_Initialise =>
			
				--SA_time_next <= to_unsigned(WD_TIME,8);
				TA_state_next <= State_Wait_1;
				WDT_O_out <= '1';
				SA_Timer_Start <= '0';
				
				if WDT_I_wdt_reset = '0' then
					TA_state_next <= State_Wait_1;
				else
					TA_state_next <= State_Initialise;
				end if;
					
			
			when State_Wait_1 =>
			
				WDT_O_out <= '1';
				if WDT_I_trigger = '1' then
					TA_state_next <= State_Trigger;
				else
					TA_state_next <= State_Wait_1;
				end if;
	
			when State_Trigger =>
						
				WDT_O_out <= '1';
				if WDT_I_trigger = '1' then
					SA_timer_start <= '1';
					TA_state_next <= State_Wait_2;
				else
					TA_state_next <= State_Initialise;
				end if;

			when State_Wait_2 =>
	
				WDT_O_out <= '1';
				if WDT_I_trigger = '1' then
				
					if SA_timer_complete = '1' then
						--TA_state_next <= State_Decrement;
						TA_state_next <= State_Time_Out;
					else
						TA_State_Next <= State_Wait_2;
					end if;
					
				else
					TA_state_next <= State_Initialise;
				end if;


--			when State_Decrement =>
--			
--				WDT_O_out <= '1';
--				if WDT_I_trigger = '1' then
--				
--					if SA_Time_Current /= 0 then
--						SA_Time_next <= SA_Time_Current-1;
--						TA_state_next <= State_Trigger;
--					else
--						TA_state_next <= State_Time_Out;
--					end if;
--					
--				else
--					TA_state_next <= State_Initialise;
--				end if;
				
			when State_Time_Out =>
			
				WDT_O_out <= '0';
				
				if (WDT_I_wdt_reset = '1' and WDT_I_Trigger = '0') then
					TA_state_next <= State_Initialise;
				else
					TA_state_next <= State_Time_Out;
				end if;


		end case;
	end process;

end arch;

