-----------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.STD_LOGIC_UNSIGNED.all;
use work.sercon_types.all;
-----------------------------------------------------------


-- Entity
-----------------------------------------------------------
entity sercon_block is
	generic(
		--
		-- NB: ensure n_regs >= 2
		--
		n_regs		: natural := 3;
		bit_width	: natural := sercon_bit_width
	);
	port(
		ctrl		: in	sercon_ctrl;
		--
		--
		sbus_in	: in	sercon_bus;
		sbus_out	: out	sercon_bus;
		--
		-- Parallel data output
		--
		pdout		: out	long_word_array(n_regs - 1 downto 0)
	);
end entity sercon_block;

-- Architecture
-----------------------------------------------------------
architecture v0 of sercon_block is
	
	-- Components
	-----------------------------
	
	component sercon_reg is
		generic(
			reset_value	: integer;
			bit_width	: natural
		);
		port(
			ctrl		: in	sercon_ctrl;
			--
			sbus_in	: in	sercon_bus;
			sbus_out	: out	sercon_bus;
			--
			pdout		: out	std_logic_vector(bit_width - 1 downto 0);
			pdin		: in	std_logic_vector(bit_width - 1 downto 0) := (others => '0')
		);
	end component sercon_reg;
	
	-- Signals
	-----------------------------
	
	signal int_pdout	: long_word_array(n_regs - 1 downto 0);
	signal int_sbus	: sercon_bus_array(n_regs - 1 downto 0);
	
begin
	
	-- Generate statements
	-----------------------------
	
	gen_rarray: for i in 0 to (n_regs - 1) generate
		
		u_reg0: if(i = 0) generate 
			u_idx0: sercon_reg
				generic map(
					reset_value	=> 0,
					bit_width	=> bit_width
				)
				port map(
					ctrl		=> ctrl,
					sbus_in	=> sbus_in,
					sbus_out	=> int_sbus(0),
					pdout		=> int_pdout(0),
					pdin		=> int_pdout(0)
				);
		end generate u_reg0;
		
		u_regs: if(i > 0 and i < (n_regs - 1)) generate
			u_idx: sercon_reg
				generic map(
					reset_value	=> 0,
					bit_width	=> bit_width
				)
				port map(
					ctrl		=> ctrl,
					sbus_in	=> int_sbus(i - 1),
					sbus_out	=> int_sbus(i),
					pdout		=> int_pdout(i),
					pdin		=> int_pdout(i)
				);
		end generate u_regs;
		
		u_regn: if(i = (n_regs - 1)) generate 
			u_idxn: sercon_reg
				generic map(
					reset_value	=> 0,
					bit_width	=> bit_width
				)
				port map(
					ctrl		=> ctrl,
					sbus_in	=> int_sbus(i - 1),
					sbus_out	=> sbus_out,
					pdout		=> int_pdout(n_regs - 1),
					pdin		=> int_pdout(n_regs - 1)
				);
		end generate u_regn;
		
	end generate gen_rarray;
	
	-- Concurrent signal
	-- assignment
	-----------------------------
	
	pdout <= int_pdout;
	
end architecture v0;
